@@ -3,23 +3,23 @@
 
 import unittest
 import sys
-sys.path.append("./src/")
-# import pdb
-# pdb.set_trace();
+
 
 import scipy.io
 import math
 import copy
-
-import logging
-logger = logging.getLogger(__name__)
-
-
 import argparse
 import numpy as np
 import matplotlib.pyplot as plt
 import matplotlib
 from matplotlib.widgets import Slider, Button
+import traceback
+
+import logging
+logger = logging.getLogger(__name__)
+
+# import pdb
+# pdb.set_trace();
 
 try:
     from PyQt4 import QtGui, QtCore
@@ -31,7 +31,6 @@ except:
 
 
 class sed3:
-
     """ Viewer and seed editor for 2D and 3D data.
 
     sed3(img, ...)
@@ -57,13 +56,12 @@ class sed3:
     # must be same')
 
     def __init__(
-        self, img, voxelsizemm=[1, 1, 1], initslice=0, colorbar=True,
-        cmap=matplotlib.cm.Greys_r, seeds=None, contour=None, zaxis=0,
-        mouse_button_map={1: 1, 2: 2, 3: 3, 4: 4, 5: 5, 6: 6, 7: 7, 8: 8},
-        windowW=[], windowC=[], show=False, sed3_on_close=None, figure=None
+            self, img, voxelsizemm=[1, 1, 1], initslice=0, colorbar=True,
+            cmap=matplotlib.cm.Greys_r, seeds=None, contour=None, zaxis=0,
+            mouse_button_map={1: 1, 2: 2, 3: 3, 4: 4, 5: 5, 6: 6, 7: 7, 8: 8},
+            windowW=None, windowC=None, show=False, sed3_on_close=None, figure=None
     ):
 
-        
         self.sed3_on_close = sed3_on_close
         self.show_fcn = plt.show
         if figure is None:
@@ -99,12 +97,7 @@ class sed3:
             self.seeds = np.zeros(self.imgshape, np.int8)
         else:
             self.seeds = seeds
-        if not (windowW and windowC):
-            self.imgmax = np.max(img)
-            self.imgmin = np.min(img)
-        else:
-            self.imgmax = windowC + (windowW / 2)
-            self.imgmin = windowC - (windowW / 2)
+        self.set_window(windowC, windowW)
 
         """ Mapping mouse button to class number. Default is normal order"""
         self.button_map = mouse_button_map
@@ -114,57 +107,202 @@ class sed3:
         self.press = None
         self.press2 = None
 
-# language
-        self.texts = {'btn_delete': 'Delete', 'btn_close': 'Close'}
+        # language
+        self.texts = {'btn_delete': 'Delete', 'btn_close': 'Close',
+                      'btn_view0': "v0", 'btn_view1': "v1", 'btn_view2': "v2"}
 
         # iself.fig.subplots_adjust(left=0.25, bottom=0.25)
-        self.ax = self.fig.add_axes([0.2, 0.3, 0.7, 0.6])
+        self.ax = self.fig.add_axes([0.1, 0.20, 0.8, 0.75])
+        # self.ax.set_axis_off()
+        # self.ax.axis('off')
 
+        self.ax_colorbar = self.fig.add_axes([0.9, 0.30, 0.02, 0.6])
         self.draw_slice()
 
         if self.colorbar:
-            self.fig.colorbar(self.imsh)
+            # self.colorbar_obj = self.fig.colorbar(self.imsh)
+            self.colorbar_obj = plt.colorbar(self.imsh, cax=self.ax_colorbar)
+            try:
+                self.colorbar_obj.on_mappable_changed(self.imsh)
+            except:
+                traceback.print_exc()
+                logger.warning("with old matplotlib version does not work colorbar redraw")
 
         # user interface look
 
         axcolor = 'lightgoldenrodyellow'
-        ax_actual_slice = self.fig.add_axes(
-            [0.2, 0.2, 0.6, 0.03], axisbg=axcolor)
-        self.actual_slice_slider = Slider(ax_actual_slice, 'Slice', 0,
+        self.ax_actual_slice = self.fig.add_axes(
+            [0.15, 0.15, 0.7, 0.03], axisbg=axcolor)
+        self.actual_slice_slider = Slider(self.ax_actual_slice, 'Slice', 0,
                                           self.imgshape[2] - 1,
                                           valinit=initslice)
 
+        immax = np.max(self.img)
+        immin = np.min(self.img)
+        # axcolor_front = 'darkslategray'
+        ax_window_c = self.fig.add_axes(
+            [0.5, 0.05, 0.2, 0.02], axisbg=axcolor)
+        self.window_c_slider = Slider(ax_window_c, 'Center',
+                                          immin,
+                                          immax,
+                                          valinit=float(self.windowC))
+        ax_window_w = self.fig.add_axes(
+            [0.5, 0.10, 0.2, 0.02], axisbg=axcolor)
+        self.window_w_slider = Slider(ax_window_w, 'Width',
+                                      0,
+                                      (immax - immin) * 2,
+                                      valinit=float(self.windowW))
         # conenction to wheel events
         self.fig.canvas.mpl_connect('scroll_event', self.on_scroll)
         self.actual_slice_slider.on_changed(self.sliceslider_update)
-# draw
+        self.window_c_slider.on_changed(self._on_window_c_change)
+        self.window_w_slider.on_changed(self._on_window_w_change)
+        # draw
         self.fig.canvas.mpl_connect('button_press_event', self.on_press)
         self.fig.canvas.mpl_connect('button_release_event', self.on_release)
         self.fig.canvas.mpl_connect('motion_notify_event', self.on_motion)
 
-# delete seeds
-        self.ax_delete_seeds = self.fig.add_axes([0.2, 0.1, 0.1, 0.075])
+        # delete seeds
+        self.ax_delete_seeds = self.fig.add_axes([0.05, 0.05, 0.1, 0.075])
         self.btn_delete = Button(
             self.ax_delete_seeds, self.texts['btn_delete'])
         self.btn_delete.on_clicked(self.callback_delete)
 
-# close button
-        self.ax_delete_seeds = self.fig.add_axes([0.7, 0.1, 0.1, 0.075])
+        # close button
+        self.ax_delete_seeds = self.fig.add_axes([0.85, 0.05, 0.1, 0.075])
         self.btn_delete = Button(self.ax_delete_seeds, self.texts['btn_close'])
         self.btn_delete.on_clicked(self.callback_close)
 
+        # im shape
+        # self.ax_shape = self.fig.add_axes([0.85, 0.05, 0.1, 0.075])
+        self.fig.text(0.20, 0.10, 'shape')
+        sh = self.img.shape
+        self.fig.text(0.20, 0.05, "%ix%ix%i" % (sh[0], sh[1], sh[2]))
         self.draw_slice()
+
+
+        # view0
+        self.ax_v0= self.fig.add_axes([0.05, 0.80, 0.08, 0.075])
+        self.btn_v0 = Button(
+            self.ax_v0, self.texts['btn_view0'])
+        self.btn_v0.on_clicked(self._callback_v0)
+
+        # view1
+        self.ax_v1= self.fig.add_axes([0.05, 0.70, 0.08, 0.075])
+        self.btn_v1 = Button(
+            self.ax_v1, self.texts['btn_view1'])
+        self.btn_v1.on_clicked(self._callback_v1)
+
+        # view2
+        self.ax_v2= self.fig.add_axes([0.05, 0.60, 0.08, 0.075])
+        self.btn_v2 = Button(
+            self.ax_v2, self.texts['btn_view2'])
+        self.btn_v2.on_clicked(self._callback_v2)
         if show:
             self.show()
 
+    def _callback_v0(self, event):
+        self.rotate_to_zaxis(0)
+
+    def _callback_v1(self, event):
+        self.rotate_to_zaxis(1)
+
+    def _callback_v2(self, event):
+        self.rotate_to_zaxis(2)
+
+    def set_window(self, windowC, windowW):
+        """
+        Sets visualization window
+        :param windowC: window center
+        :param windowW: window width
+        :return:
+        """
+        if not (windowW and windowC):
+            windowW = np.max(self.img) - np.min(self.img)
+            windowC = (np.max(self.img) + np.min(self.img)) / 2.0
+
+        self.imgmax = windowC + (windowW / 2)
+        self.imgmin = windowC - (windowW / 2)
+        self.windowC = windowC
+        self.windowW = windowW
+
+            # self.imgmax = np.max(self.img)
+            # self.imgmin = np.min(self.img)
+            # self.windowC = windowC
+            # self.windowW = windowW
+        # else:
+        #     self.imgmax = windowC + (windowW / 2)
+        #     self.imgmin = windowC - (windowW / 2)
+        #     self.windowC = windowC
+        #     self.windowW = windowW
+
+    def _on_window_w_change(self, windowW):
+        self.set_window(self.windowC, windowW)
+        if self.colorbar:
+            try:
+                self.colorbar_obj.on_mappable_changed(self.imsh)
+            except:
+                traceback.print_exc()
+                logger.warning("with old matplotlib version does not work colorbar redraw")
+
+        self.update_slice()
+
+    def _on_window_c_change(self, windowC):
+        self.set_window(windowC, self.windowW)
+        if self.colorbar:
+            try:
+                self.colorbar_obj.on_mappable_changed(self.imsh)
+            except:
+                traceback.print_exc()
+                logger.warning("with old matplotlib version does not work colorbar redraw")
+
+        self.update_slice()
+
+    def rotate_to_zaxis(self, new_zaxis):
+        """
+        rotate image to selected axis
+        :param new_zaxis:
+        :return:
+        """
+
+        img = self._rotate_end(self.img, self.zaxis)
+        seeds = self._rotate_end(self.seeds, self.zaxis)
+        contour = self._rotate_end(self.contour, self.zaxis)
+
+        # Rotate data in depndecy on zaxispyplot
+        self.img = self._rotate_start(img, new_zaxis)
+        self.seeds = self._rotate_start(seeds, new_zaxis)
+        self.contour = self._rotate_start(contour, new_zaxis)
+        self.zaxis = new_zaxis
+        # import ipdb
+        # ipdb.set_trace()
+        # self.actual_slice_slider.valmax = self.img.shape[2] - 1
+        self.actual_slice = 0
+        self.rotated_back = False
+
+        # update slicer
+        self.fig.delaxes(self.ax_actual_slice)
+        self.ax_actual_slice.cla()
+        del(self.actual_slice_slider)
+        self.fig.add_axes(self.ax_actual_slice)
+        self.actual_slice_slider = Slider(self.ax_actual_slice, 'Slice', 0,
+                                          self.img.shape[2] - 1,
+                                          valinit=0)
+        self.actual_slice_slider.on_changed(self.sliceslider_update)
+        self.update_slice()
+
     def _rotate_start(self, data, zaxis):
         if data is not None:
             if zaxis == 0:
                 data = np.transpose(data, (1, 2, 0))
+            elif zaxis == 1:
+                data = np.transpose(data, (2, 0, 1))
             elif zaxis == 2:
+                # data = np.transpose(data, (0, 1, 2))
                 pass
             else:
                 print "problem with zaxis in _rotate_start()"
+                logger.warning("problem with zaxis in _rotate_start()")
 
         return data
 
@@ -173,13 +311,17 @@ class sed3:
             if self.rotated_back is False:
                 if zaxis == 0:
                     data = np.transpose(data, (2, 0, 1))
+                elif zaxis == 1:
+                    data = np.transpose(data, (1, 2, 0))
                 elif zaxis == 2:
                     pass
                 else:
                     print "problem with zaxis in _rotate_start()"
+                    logger.warning("problem with zaxis in _rotate_start()")
 
             else:
                 print "There is a danger in calling show() twice"
+                logger.warning("There is a danger in calling show() twice")
 
         return data
 
@@ -213,6 +355,11 @@ class sed3:
             except:
                 pass
 
+        # self.ax.set_axis_off()
+        self.ax.set_xticklabels([])
+        self.ax.set_yticklabels([])
+        # self.ax.set_axis_below(False)
+
         # print ctr
         # import pdb; pdb.set_trace()
 
@@ -267,7 +414,7 @@ class sed3:
 
     def prepare_output_data(self):
         if self.rotated_back is False:
-        # Rotate data in depndecy on zaxis
+            # Rotate data in depndecy on zaxis
             self.img = self._rotate_end(self.img, self.zaxis)
             self.seeds = self._rotate_end(self.seeds, self.zaxis)
             self.contour = self._rotate_end(self.contour, self.zaxis)
@@ -285,7 +432,7 @@ class sed3:
         # self.show_slice()
         # print self.actual_slice
 
-# malování -------------------
+    # malování -------------------
     def on_press(self, event):
         'on but-ton press we will see if the mouse is over us and store data'
         if event.inaxes != self.ax:
@@ -350,7 +497,8 @@ class sed3:
         for i, item in enumerate(px):
             self.seeds[int(item), int(py[i]), int(pz)] = value
 
-# @todo
+        # @todo
+
     def get_seed_sub(self, label):
         """ Return list of all seeds with specific label
         """
@@ -381,19 +529,19 @@ def show_slices(data3d, contour=None, seeds=None, axis=0, slice_step=1,
 
     number_of_slices = data3d.shape[axis]
     # square image
-#     nn = int(math.ceil(number_of_slices ** 0.5))
+    #     nn = int(math.ceil(number_of_slices ** 0.5))
 
-#     sh = [nn, nn]
+    #     sh = [nn, nn]
 
     # 4:3 image
     sh = shape
     if sh is None:
-        na = int(math.ceil(number_of_slices * 16.0 / 9.0)**0.5)
+        na = int(math.ceil(number_of_slices * 16.0 / 9.0) ** 0.5)
         nb = int(math.ceil(float(number_of_slices) / na))
         sh = [nb, na]
 
     dsh = __get_slice(data3d, 0, axis).shape
-    slimsh = [int(dsh[0]*sh[0]), int(dsh[1] * sh[1])]
+    slimsh = [int(dsh[0] * sh[0]), int(dsh[1] * sh[1])]
     slim = np.zeros(slimsh, dtype=data3d.dtype)
     slco = None
     slse = None
@@ -401,8 +549,8 @@ def show_slices(data3d, contour=None, seeds=None, axis=0, slice_step=1,
         slse = np.zeros(slimsh, dtype=seeds.dtype)
     if contour is not None:
         slco = np.zeros(slimsh, dtype=contour.dtype)
-#         slse =
-#     f, axarr = plt.subplots(sh[0], sh[1])
+    #         slse =
+    #     f, axarr = plt.subplots(sh[0], sh[1])
 
     for i in range(0, number_of_slices):
         cont = None
@@ -414,18 +562,18 @@ def show_slices(data3d, contour=None, seeds=None, axis=0, slice_step=1,
         if seeds is not None:
             seeds2d = __get_slice(seeds, i, axis)
             slse = __put_slice_in_slim(slse, seeds2d, sh, i)
-#         plt.axis('off')
-#         plt.subplot(sh[0], sh[1], i+1)
-#         plt.subplots_adjust(wspace=0, hspace=0)
+        #         plt.axis('off')
+        #         plt.subplot(sh[0], sh[1], i+1)
+        #         plt.subplots_adjust(wspace=0, hspace=0)
 
         slim = __put_slice_in_slim(slim, im2d, sh, i)
-#         show_slice(im2d, cont, seeds2d)
-    plt.axis('off')
+    #         show_slice(im2d, cont, seeds2d)
     show_slice(slim, slco, slse)
     if show:
         plt.show()
 
-#         a, b = np.unravel_index(i, sh)
+
+# a, b = np.unravel_index(i, sh)
 
 #     pass
 
@@ -438,6 +586,7 @@ def __get_slice(data, slice_number, axis=0):
     elif axis == 2:
         return data[:, :, slice_number]
     else:
+        logger.error("axis number error")
         print "axis number error"
         return None
 
@@ -454,8 +603,8 @@ def __put_slice_in_slim(slim, dataim, sh, i):
     sp1 = int(st1 + dataim.shape[1])
 
     slim[
-        st0:sp0,
-        st1:sp1
+    st0:sp0,
+    st1:sp1
     ] = dataim
 
     return slim
@@ -515,14 +664,13 @@ def _import_data(data, axis, slice_step):
 
 # self.rect.figure.canvas.draw()
 
-    # return data
+# return data
 
 class sed3qt(QtGui.QDialog):
     def __init__(self, *pars, **params):
-    # def __init__(self,parent=None):
+        # def __init__(self,parent=None):
         parent = None
 
-
         QtGui.QDialog.__init__(self, parent)
         # super(Window, self).__init__(parent)
         # self.setupUi(self)
@@ -530,17 +678,17 @@ class sed3qt(QtGui.QDialog):
         self.canvas = FigureCanvas(self.figure)
         self.toolbar = NavigationToolbar(self.canvas, self)
 
-# set the layout
+        # set the layout
         layout = QtGui.QVBoxLayout()
         layout.addWidget(self.toolbar)
         layout.addWidget(self.canvas)
         # layout.addWidget(self.button)
         self.setLayout(layout)
 
-    # def set_params(self, *pars, **params):
+        # def set_params(self, *pars, **params):
         # import sed3.sed3
 
-        params["figure"] = self.figure 
+        params["figure"] = self.figure
         self.sed = sed3(*pars, **params)
         self.sed.sed3_on_close = self.callback_close
         # ed.show()
@@ -558,9 +706,9 @@ class sed3qt(QtGui.QDialog):
     def get_values(self):
         return self.sed
 
+
 # --------------------------tests-----------------------------
 class Tests(unittest.TestCase):
-
     def test_t(self):
         pass
 
@@ -635,20 +783,21 @@ def generate_data(shp=[16, 20, 24]):
     """ Generating data """
 
     x = np.ones(shp)
-# inserting box
+    # inserting box
     x[4:-4, 6:-2, 1:-6] = -1
     x_noisy = x + np.random.normal(0, 0.6, size=x.shape)
     return x_noisy
 
+
 # --------------------------main------------------------------
 if __name__ == "__main__":
     logger = logging.getLogger()
     logger.setLevel(logging.WARNING)
-# při vývoji si necháme vypisovat všechny hlášky
+    # při vývoji si necháme vypisovat všechny hlášky
     # logger.setLevel(logging.DEBUG)
 
     ch = logging.StreamHandler()
-#   output configureation
+    #   output configureation
     # logging.basicConfig(format='%(asctime)s %(message)s')
     logging.basicConfig(format='%(message)s')
 
@@ -693,6 +842,7 @@ if __name__ == "__main__":
         data = generate_data()
     elif args.filename == 'lena':
         from scipy import misc
+
         data = misc.lena()
     else:
         #   load all
@@ -719,3 +869,68 @@ if __name__ == "__main__":
 
     scipy.io.savemat(args.outputfile, {'data': output})
     pyed.get_seed_val(1)
+
+
+def index_to_coords(index, shape):
+    '''convert index to coordinates given the shape'''
+    coords = []
+    for i in xrange(1, len(shape)):
+        divisor = int(np.product(shape[i:]))
+        value = index // divisor
+        coords.append(value)
+        index -= value * divisor
+    coords.append(index)
+    return tuple(coords)
+
+
+sh = np.asarray([3, 4])
+
+
+def slices(img, shape=[3, 4]):
+    """
+    create tiled image with multiple slices
+    :param img:
+    :param shape:
+    :return:
+    """
+    sh = np.asarray(shape)
+    i_max = np.prod(sh)
+    allimg = np.zeros(img.shape[-2:] * sh)
+
+    for i in range(0, i_max):
+        # i = 0
+        islice = round((img.shape[0] / float(i_max)) * i)
+        #         print islice
+        imgi = img[islice, :, :]
+        coords = index_to_coords(i, sh)
+        aic = np.asarray(img.shape[-2:]) * coords
+
+        allimg[aic[0]:aic[0] + imgi.shape[-2], aic[1]:aic[1] + imgi.shape[-1]] = imgi
+
+    #     plt.imshow(imgi)
+    #     print imgi.shape
+    #     print img.shape
+    return allimg
+
+
+# sz = img.shape
+# np.zeros()
+def sed2(img, contour=None, shape=[3, 4]):
+    """
+    plot tiled image of multiple slices
+
+    :param img:
+    :param contour:
+    :param shape:
+    :return:
+    """
+    """
+    :param img:
+    :param contour:
+    :param shape:
+    :return:
+    """
+
+    plt.imshow(slices(img, shape), cmap='gray')
+    if contour is not None:
+        plt.contour(slices(contour, shape))
